/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import org.w3c.dom.Node;

/**
 * &lt;checkbox&gt;<br>
  * Children (besides the inherited):<br>
 *   labelPos
 *   onValue
 *   offValue
 */
public class CheckBoxField extends A_SingleValueField 
{
	private String labelPos;
	private String onValue;
	private String offValue;
	
	/**
	 * @param form
	 * @param type
	 * @param cardinality
	 */
	public CheckBoxField(Form form) {
		super(form);
	}
	/**
	 * @param form
	 * @param field
	 * @param cardinality of values, is multiple for radio.
	 */
	public CheckBoxField(Form form, Node field) {
		super(form, field);
	}
	
	public String tagName() {
		return "checkbox";
	}
	
	/**
	 * Look for relevant sub-elements
	 * @param child
	 * @param childType
	 * @param childName
	 * @return true if child is considered handled, false if it didn't cause any changes
	 */
	public boolean handleChildNode(Node child, short childType, String childName)
	{
		if(super.handleChildNode(child, childType, childName))
			return true;
		if(childType == Node.ELEMENT_NODE) {
			if (childName.equals(Form.FBTAG_LABEL_POS)) {
				setLabelPos(Form.collectText(child));
				return true;
			}
			else if (childName.equals(Form.FBTAG_ON_VALUE)) {
				setOnValue(Form.collectText(child));
				return true;
			}
			else if (childName.equals(Form.FBTAG_OFF_VALUE)) {
				setOffValue(Form.collectText(child));
				return true;
			}
		}
		return false;
	}
	
	public String getValue() {
		if(super.getValue() == null)
			return getOffValue();
		else
			return super.getValue();
	}
	
	public void setValue(String str) {
		if(str == null)
			super.setValue(getOffValue());
		else if (str.equals(getOnValue()) || str.equals(getOffValue()))
			super.setValue(str);
		else
			super.setValue(getOnValue());
	}
	
	public String getOnValue() {
		return (onValue == null ? "true" : onValue);
	}
	
	public void setOnValue(String str) {
		onValue = (str == null ? "true" : str); // off may be null, this may not!
	}

	public String getOffValue() {
		return (offValue == null ? "false" : offValue);
	}
	
	public void setOffValue(String str) {
		this.offValue = str;
	}
	
	public String getLabelPos() {
		return labelPos;
	}
	
	public void setLabelPos(String labelPos) {
		this.labelPos = labelPos;
	}

	public String childrenToXML(String indent, boolean inclVal) 
	{
		String tmp = super.childrenToXML(indent, inclVal);
	    if(tmp == null && getValue() == null)
	    	return null;
		StringBuffer b = new StringBuffer();
		if(tmp != null)
			b.append(tmp);
		if(getLabelPos() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_LABEL_POS+">" + getLabelPos() + "</"+Form.FBTAG_LABEL_POS+">");
		if(getOnValue() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_ON_VALUE+">" + getOnValue() + "</"+Form.FBTAG_ON_VALUE+">");
		if(getOffValue() != null)
			b.append("\n" + indent + "<"+Form.FBTAG_OFF_VALUE+">" + getOffValue() + "</"+Form.FBTAG_OFF_VALUE+">");
 		return b.toString();
	}
}
