/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.forms;

import java.util.HashMap;
import java.util.Iterator;

import org.w3c.dom.*;

/**
 *
 */
public class Option 
{
    private String tagName;
	private HashMap labels;
	private String value;
	protected Form form;

	public Option(Form form) {
		super();
		this.form = form;
		this.tagName = Form.FBTAG_OPTION;
	}
	
	public Option(Form form, Node opt) {
		super();
		this.form = form;
		populateFromNode(opt);
	}
	
	public String tagName() {
		return tagName;
	}
	
	public boolean isGroup() {
		return (Form.FBTAG_GROUP.equals(tagName()));
	}
	
	public void populateFromNode(Node opt) 
	{
		tagName = opt.getNodeName();

		if(Form.FBTAG_OPTION.equals(tagName())) {
			NodeList list = opt.getChildNodes();
			int len = list.getLength();
			boolean gotLbl = false, gotVal = false;
			for(int i = 0; i < len; i++) {
				Node c = list.item(i);
				if(c.getNodeType() == Node.ELEMENT_NODE){
					if(Form.FBTAG_LABEL.equals(c.getNodeName())) {
						String lbl = Form.collectText(c);
						String loc = "";
						NamedNodeMap attrs = c.getAttributes();
						for(int ii=0; ii<attrs.getLength(); ii++) {
							Node attr = attrs.item(ii);
							String an = attr.getNodeName();
							if(an.equals(Form.FBATTR_LOCALE)) {
								loc = Form.collectText(attr);
							}
						}
						setLabel(lbl, loc);
						gotLbl = true; // got at least one...
					} else if(Form.FBTAG_VALUE.equals(c.getNodeName())) {
						setValue(Form.collectText(c));
						gotVal = true;
					}
				}
			}
			if(gotLbl && !gotVal) {
				setValue(getLabel("")); // use a label as value
			}
			else if(!gotLbl && gotVal) {
				setLabel(getValue(), ""); // use value as label
			}
			else if(!gotLbl && !gotVal) {
				String txt = Form.collectText(opt); // on the form <option>foobar</option>
				setLabel(txt, "");
				setValue(txt);
			}
			// else we got both so we're done
		}	
		// else if FBTAG_GROUP
	}
	
	//
	//
	//
	
	/**
	 * First it checks the label based on the current locale of the form, if that fails
	 * it tries one more time with the "" locale (i.e. labels not specifically marked with locale
	 * @return
	 */
	public String getLabel() {
		String retval = getLabel(form.getLocale());
		if(retval==null)
			retval = (String)labels.get("");
		return retval;
	}
	
	public void setLabel(String str) {
		setLabel(str, "");
	}

	public String getLabel(String locale) {
		if(labels != null) {
			return (String)labels.get(locale);
		} else {
			return null;
		}
	}

	public void setLabel(String str, String locale) {
		if(labels == null)
			labels = new HashMap();
		labels.put(locale, str);
	}
	
	public String getValue() {
		return value;
	}
	
	public void setValue(String value) {
		this.value = value;
	}

	//
	//
	//
	
	public String toXML(String indent) {
		return "\n" + indent + "<"+tagName()+">" + childrenToXML(indent + Form.PP_INDENT) + "\n" + indent + "</"+tagName()+">";
	}
	
	public String childrenToXML(String indent) 
	{
		StringBuffer b = new StringBuffer();
		if(!isGroup()) {
			if(labels != null) {
				Iterator it = labels.keySet().iterator();
				while(it.hasNext()) {
					String loc = (String)it.next();
					if("".equals(loc)) {
						b.append("\n" + indent + "<"+Form.FBTAG_LABEL+">" + labels.get(loc) + "</"+Form.FBTAG_LABEL+">");
					} else {
						b.append("\n" + indent + "<"+Form.FBTAG_LABEL+" "+Form.FBATTR_LOCALE+"=\"" +loc+ "\">" + labels.get(loc) + "</"+Form.FBTAG_LABEL+">");
					}
				}
			}
			if(getValue() != null)
				b.append("\n" + indent + "<"+Form.FBTAG_VALUE+">" + getValue() + "</"+Form.FBTAG_VALUE+">");			
		}
		return b.toString();
	}
}
