/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 180 $
 * $LastChangedDate: 2007-06-20 11:22:10 +0200 (Wed, 20 Jun 2007) $
 * 
 */

package com.codedroids.records;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.sql.Blob;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;

import com.codedroids.j.StringUtils;

public class RecordField 
{
	private static Map resultClasses;
	private static Map resultTypes;
	private static Map resultGetters;
	private static Map resultSetters;
	
	static {
		resultClasses = new HashMap();
		resultTypes = new HashMap(); // only needed because setNull needs the column type...
		resultGetters = new HashMap();
		resultSetters = new HashMap();
		Class rsc = ResultSet.class;
		Class psc = PreparedStatement.class;
		Class[] getpar = new Class[] { int.class }; // or use the column name 
		try {
			// Array
			// AsciiStream
			// BigDecimal
			resultClasses.put("BigDecimal", BigDecimal.class );
			resultTypes.put("BigDecimal", new Integer(Types.NUMERIC));
			resultGetters.put("BigDecimal", rsc.getMethod("getBigDecimal", getpar ));
			resultSetters.put("BigDecimal", psc.getMethod("setBigDecimal", new Class[] { int.class, BigDecimal.class } ));
			// BinaryStream
			// Blob
			resultClasses.put("Blob", Blob.class );
			resultTypes.put("Blob", new Integer(Types.BLOB));
			resultGetters.put("Blob", rsc.getMethod("getBlob", getpar ));
			resultSetters.put("Blob", psc.getMethod("setBlob", new Class[] { int.class, Blob.class } ));		
			// boolean
			resultClasses.put("boolean", boolean.class );
			resultTypes.put("boolean", new Integer(Types.BIT));
			resultGetters.put("boolean", rsc.getMethod("getBoolean", getpar ));
			resultSetters.put("boolean", psc.getMethod("setBoolean", new Class[] { int.class, boolean.class } ));		
			// Boolean
			resultClasses.put("Boolean", Boolean.class );
			resultTypes.put("Boolean", new Integer(Types.BIT));
			resultGetters.put("Boolean", rsc.getMethod("getBoolean", getpar ));
			resultSetters.put("Boolean", psc.getMethod("setBoolean", new Class[] { int.class, boolean.class } ));		
			// Byte
			// Bytes
			// CharacterStream
			// Clob
			// Date
			resultClasses.put("Date", java.sql.Date.class );
			resultTypes.put("Date", new Integer(Types.DATE));
			resultGetters.put("Date", rsc.getMethod("getDate", getpar ));
			resultSetters.put("Date", psc.getMethod("setDate", new Class[] { int.class, java.sql.Date.class } ));				
			// double
			resultClasses.put("double", double.class );
			resultTypes.put("double", new Integer(Types.DOUBLE));
			resultGetters.put("double", rsc.getMethod("getDouble", getpar ));
			resultSetters.put("double", psc.getMethod("setDouble", new Class[] { int.class, double.class } ));	
			// Double
			resultClasses.put("Double", Double.class );
			resultTypes.put("Double", new Integer(Types.DOUBLE));
			resultGetters.put("Double", rsc.getMethod("getDouble", getpar ));
			resultSetters.put("Double", psc.getMethod("setDouble", new Class[] { int.class, double.class } ));	
			// float
			resultClasses.put("float", float.class );
			resultTypes.put("float", new Integer(Types.FLOAT));
			resultGetters.put("float", rsc.getMethod("getFloat", getpar ));
			resultSetters.put("float", psc.getMethod("setFloat", new Class[] { int.class, float.class } ));		
			// Float
			resultClasses.put("Float", Float.class );
			resultTypes.put("Float", new Integer(Types.FLOAT));
			resultGetters.put("Float", rsc.getMethod("getFloat", getpar ));
			resultSetters.put("Float", psc.getMethod("setFloat", new Class[] { int.class, float.class } ));		
			// int
			resultClasses.put("int", int.class );
			resultTypes.put("int", new Integer(Types.INTEGER));
			resultGetters.put("int", rsc.getMethod("getInt", getpar ));
			resultSetters.put("int", psc.getMethod("setInt", new Class[] { int.class, int.class } ));		
			// Integer
			resultClasses.put("Integer", Integer.class );
			resultTypes.put("Integer", new Integer(Types.INTEGER));
			resultGetters.put("Integer", rsc.getMethod("getInt", getpar ));
			resultSetters.put("Integer", psc.getMethod("setInt", new Class[] { int.class, int.class } ));		
			// long
			resultClasses.put("long", long.class );
			resultTypes.put("long", new Integer(Types.BIGINT));
			resultGetters.put("long", rsc.getMethod("getLong", getpar ));
			resultSetters.put("long", psc.getMethod("setLong", new Class[] { int.class, long.class } ));
			// Long
			resultClasses.put("Long", Long.class );
			resultTypes.put("Long", new Integer(Types.BIGINT));
			resultGetters.put("Long", rsc.getMethod("getLong", getpar ));
			resultSetters.put("Long", psc.getMethod("setLong", new Class[] { int.class, long.class } ));
			// Object
			// Ref
			// short
			resultClasses.put("short", short.class );
			resultTypes.put("short", new Integer(Types.SMALLINT));
			resultGetters.put("short", rsc.getMethod("getShort", getpar ));
			resultSetters.put("short", psc.getMethod("setShort", new Class[] { int.class, short.class } ));		
			// Short
			resultClasses.put("Short", Short.class );
			resultTypes.put("Short", new Integer(Types.SMALLINT));
			resultGetters.put("Short", rsc.getMethod("getShort", getpar ));
			resultSetters.put("Short", psc.getMethod("setShort", new Class[] { int.class, short.class } ));		
			// String
			resultClasses.put("String", String.class );
			resultTypes.put("String", new Integer(Types.VARCHAR));
			resultGetters.put("String", rsc.getMethod("getString", getpar ));
			resultSetters.put("String", psc.getMethod("setString", new Class[] { int.class, String.class } ));
			// Time
			resultClasses.put("Time", Time.class );
			resultTypes.put("Time", new Integer(Types.TIME));
			resultGetters.put("Time", rsc.getMethod("getTime", getpar ));
			resultSetters.put("Time", psc.getMethod("setTime", new Class[] { int.class, Time.class } ));
			// Timestamp
			resultClasses.put("Timestamp", Timestamp.class );
			resultTypes.put("Timestamp", new Integer(Types.TIMESTAMP));
			resultGetters.put("Timestamp", rsc.getMethod("getTimestamp", getpar ));
			resultSetters.put("Timestamp", psc.getMethod("setTimestamp", new Class[] { int.class, Timestamp.class } ));
			//URL
			
		} catch (NoSuchMethodException e) {
			// shouldn't happen so ignore.
			System.err.println("ContentDef result init exception: " + e );
		}
	}
	
	// TODO These are hacks until formatting in the form works
	private SimpleDateFormat defaultTimeFormatter = new SimpleDateFormat("HH:mm");
	private SimpleDateFormat defaultTimestampFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm");
	
	//
	// Instance
	//
	
	private String name;
	private String type;
	private String column;
	
	public RecordField() {
		super();
	}

	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	public String getColumn() {
		return column;
	}
	public void setColumn(String column) {
		this.column = column;
	}
	
	public String getType() {
		return type;
	}
	public void setType(String type) {
		this.type = type;
	}

	public Class fieldClass() {
		return (Class)resultClasses.get(this.type);
	}
	
	// These 2 converter methods should be streamlined to avoid the multiple string compares
	public Object fromString(String v) throws ParseException
	{	
		String t = getType();
		// Array
		// AsciiStream
		if("BigDecimal".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : new BigDecimal(v));
		}
		// BinaryStream
		// Blob
		else if("boolean".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Boolean.valueOf(v));
		}
		else if("Boolean".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Boolean.valueOf(v));
		}
		// Byte
		// Bytes
		// CharacterStream
		// Clob
		else if("Date".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : java.sql.Date.valueOf(v));
		}
		else if("double".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Double.valueOf(v));
		}
		else if("Double".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Double.valueOf(v));
		}
		else if("float".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Float.valueOf(v));
		}
		else if("Float".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Float.valueOf(v));
		}
		else if("int".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Integer.valueOf(v));
		}
		else if("Integer".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Integer.valueOf(v));
		}
		else if("long".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Long.valueOf(v));
		}
		else if("Long".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Long.valueOf(v));
		}
		// Object
		// Ref
		else if("short".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Short.valueOf(v));
		}
		else if("Short".equals(t)) {
			return (StringUtils.n4s(v) == null ? null : Short.valueOf(v));
		}
		else if("String".equals(t)) {
			return v;
		}
		else if("Time".equals(t)) {
			if(StringUtils.n4s(v) == null)
				return null;
			else
				return new Time(defaultTimeFormatter.parse(v).getTime());
		}
		else if("Timestamp".equals(t)) {
			if(StringUtils.n4s(v) == null)
				return null;
			else
				return new Timestamp(defaultTimestampFormatter.parse(v).getTime());
		}
		//URL
		return null;
	}
	
	public String toString(Object v)
	{
		if(v==null)
			return null;
		
		String t = getType();
		// Array
		// AsciiStream
		if("BigDecimal".equals(t)) {
			return ""+v;
		}
		// BinaryStream
		// Blob
		else if("boolean".equals(t)) {
			return ""+v;
		}
		else if("Boolean".equals(t)) {
			return v.toString();
		}
		// Byte
		// Bytes
		// CharacterStream
		// Clob
		else if("Date".equals(t)) {
			return ""+v;
		}
		else if("double".equals(t)) {
			return ""+v;
		}
		else if("Double".equals(t)) {
			return v.toString();
		}
		else if("float".equals(t)) {
			return ""+v;
		}
		else if("Float".equals(t)) {
			return v.toString();
		}
		else if("int".equals(t)) {
			return ""+v;
		}
		else if("Integer".equals(t)) {
			return v.toString();
		}
		else if("long".equals(t)) {
			return ""+v;
		}
		else if("Long".equals(t)) {
			return v.toString();
		}
		// Object
		// Ref
		else if("short".equals(t)) {
			return ""+v;
		}
		else if("Short".equals(t)) {
			return v.toString();
		}
		else if("String".equals(t)) {
			return (String)v;
		}
		else if("Time".equals(t)) {
			return defaultTimeFormatter.format((Time)v);
		}
		else if("Timestamp".equals(t)) {
			return defaultTimestampFormatter.format((Timestamp)v);
		}
		//URL
		return null;
	}

	public Object getFromResultSet(ResultSet rs, int index) throws IllegalAccessException, InvocationTargetException, SQLException {
		Method mg = (Method)resultGetters.get(getType());
		Object o = mg.invoke(rs, new Object[] { new Integer(index) } );
		if(rs.wasNull())
			return null;
		else
			return o;
	}
	
	// id should either be the first or the last (latter when updating)
	public void setOnPreparedStatement(PreparedStatement ps, int index, Object o) throws SQLException, IllegalAccessException, InvocationTargetException {
		if(o != null) {
			Method ms = (Method)resultSetters.get(getType());
			ms.invoke(ps, new Object[] { new Integer(index), o } );
		} else {
			ps.setNull(index, ((Integer)resultTypes.get(getType())).intValue());
		}
	}
}
